package com.mobiledefense.verify.validate

import android.graphics.Color
import android.graphics.LightingColorFilter
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.view.View
import com.dev.mobiledefense.R
import com.mobiledefense.MobileDefenseApplication
import com.mobiledefense.verify.validate.di.VerificationValidationActivityModule
import com.mobiledefense.verify.validate.model.ObservableSmsMessage
import com.mobiledefense.verify.validate.ui.presenter.VerificationValidationViewPresenter
import com.mobiledefense.verify.validate.ui.view.VerificationValidationView
import com.mobiledefense.verify.validate.ui.view.VerificationValidationView.Action
import javax.inject.Inject

class VerificationValidationActivity : AppCompatActivity() {

    companion object {
        const val EXTRA_LINE_NUMBER = "line_number"
        const val EXTRA_ID = "id"
        const val EXTRA_UPDATE_VERIFIED_NUMBER = "extra_update_verified_number"
    }

    @Inject lateinit var presenter: VerificationValidationViewPresenter
    @Inject lateinit var view: VerificationValidationView
    @Inject lateinit var observableSmsMessage: ObservableSmsMessage

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        MobileDefenseApplication.getDependencies()
            .plusDependenciesFrom(VerificationValidationActivityModule(this, intent))
            .injectInto(this)

        setContentView(view as View)
        configureActionBar()
    }

    override fun onStart() {
        super.onStart()
        presenter.viewModelObserver = view.viewModelObserver
        observableSmsMessage.addObserver(presenter.smsMessageObserver)
    }

    override fun onResume() {
        super.onResume()
        view.pinChangesObserver = presenter.pinChangesObserver
        view.validateButtonTapsObserver = presenter.validateButtonTapsObserver
        view.requestButtonTapsObserver = presenter.requestButtonTapsObserver
        view.requestErrorDialogDismissalsObserver = presenter.requestErrorDialogDismissalsObserver
    }

    override fun onPause() {
        view.requestErrorDialogDismissalsObserver = null
        view.requestButtonTapsObserver = null
        view.validateButtonTapsObserver = null
        view.pinChangesObserver = null
        super.onPause()
    }

    override fun onStop() {
        observableSmsMessage.removeObservers()
        presenter.viewModelObserver = null
        super.onStop()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            onBackPressed()
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed() {
        presenter.backTapsObserver.next(Action.BackTap)
    }

    private fun configureActionBar() {
        setSupportActionBar(findViewById(R.id.verification_validation_toolbar))
        supportActionBar?.let { actionBar ->
            ContextCompat.getDrawable(this, R.drawable.back_arrow)?.let { drawable ->
                actionBar.setDisplayHomeAsUpEnabled(true)
                actionBar.setHomeAsUpIndicator(drawable.mutate().apply {
                    colorFilter = LightingColorFilter(Color.WHITE, Color.WHITE)
                })
            }
        }
    }
}
