package com.mobiledefense.verify.validate.di

import android.app.Activity
import android.content.Context
import android.content.Intent
import com.mobiledefense.base.api.UpdatePhoneNumberApiClient
import com.mobiledefense.base.data.CoreDataHelper
import com.mobiledefense.base.data.model.CoreMetadata
import com.mobiledefense.base.di.ActivityModule
import com.mobiledefense.base.di.ApplicationContext
import com.mobiledefense.base.helper.LogHelper
import com.mobiledefense.lean.EventsTracker
import com.mobiledefense.lean.intercom.IntercomGateway
import com.mobiledefense.protection.controller.DeductibleController
import com.mobiledefense.protection.controller.PolicyDocumentController
import com.mobiledefense.tmobile.policy.provider.TMobilePolicyProvider
import com.mobiledefense.verify.features.VerificationFeatureManagerImpl
import com.mobiledefense.verify.request.api.VerificationRequestApi
import com.mobiledefense.verify.update.UpdatePhoneNumber
import com.mobiledefense.verify.validate.VerificationValidationActivity.Companion.EXTRA_ID
import com.mobiledefense.verify.validate.VerificationValidationActivity.Companion.EXTRA_LINE_NUMBER
import com.mobiledefense.verify.validate.VerificationValidationActivity.Companion.EXTRA_UPDATE_VERIFIED_NUMBER
import com.mobiledefense.verify.validate.api.VerificationValidationApi
import com.mobiledefense.verify.validate.helper.VerificationPinValidator
import com.mobiledefense.verify.validate.helper.VerificationValidationNavigationHelperImpl
import com.mobiledefense.verify.validate.model.ObservableSmsMessage
import com.mobiledefense.verify.validate.model.ObservableSmsMessageImpl
import com.mobiledefense.verify.validate.ui.presenter.VerificationValidationViewPresenter
import com.mobiledefense.verify.validate.ui.presenter.VerificationValidationViewPresenterImpl
import com.mobiledefense.verify.validate.ui.view.VerificationValidationView
import com.mobiledefense.verify.validate.ui.view.VerificationValidationViewImpl
import dagger.Module
import dagger.Provides

@Module
class VerificationValidationActivityModule(activity: Activity,
                                           private val intent: Intent)
                                           : ActivityModule(activity) {

    @Provides
    fun providesPresenter(@ApplicationContext appContext: Context,
                          validateApi: VerificationValidationApi,
                          requestApi: VerificationRequestApi,
                          eventsTracker: EventsTracker)
                          : VerificationValidationViewPresenter {
        val featureManager = VerificationFeatureManagerImpl(CoreDataHelper.getInstance(appContext).featureManager)
        val pinValidator = VerificationPinValidator()
        val navigationHelper = VerificationValidationNavigationHelperImpl(activity)

        val lineNumber = intent.getStringExtra(EXTRA_LINE_NUMBER)
        val id = intent.getStringExtra(EXTRA_ID)

        val updatePhoneNumber = if (intent.getBooleanExtra(EXTRA_UPDATE_VERIFIED_NUMBER, false)) {
            UpdatePhoneNumber(
                CoreMetadata.getInstance(appContext),
                TMobilePolicyProvider.fromContext(appContext),
                DeductibleController.fromContext(appContext),
                PolicyDocumentController.fromContext(appContext),
                eventsTracker,
                IntercomGateway.getInstance(),
                UpdatePhoneNumberApiClient(appContext),
                featureManager,
                LogHelper(UpdatePhoneNumber::class.java))
        } else {
            null
        }

        return VerificationValidationViewPresenterImpl(featureManager, pinValidator, validateApi,
            requestApi, navigationHelper, eventsTracker, lineNumber, id, updatePhoneNumber)
    }

    @Provides
    fun providesView(): VerificationValidationView = VerificationValidationViewImpl(activity)

    @Provides
    fun providesObservableSmsMessage(@ApplicationContext appContext: Context): ObservableSmsMessage
        = ObservableSmsMessageImpl(appContext)
}
