package com.mobiledefense.verify.validate.ui.model

sealed class VerificationValidationViewModel {
    abstract val lineNumber: String
    abstract val id: String
    abstract val pin: String

    data class Input(
        override val lineNumber: String,
        override val id: String,
        override val pin: String)
        : VerificationValidationViewModel()

    data class InputError(
        override val lineNumber: String,
        override val id: String,
        override val pin: String)
        : VerificationValidationViewModel()

    data class Requesting @JvmOverloads constructor(
        override val lineNumber: String,
        override val id: String,
        override val pin: String,
        val validationFailureReason: ValidationFailure.Reason? = null)
        : VerificationValidationViewModel()

    data class RequestError @JvmOverloads constructor(
        override val lineNumber: String,
        override val id: String,
        override val pin: String,
        val message: String? = null,
        val validationFailureReason: ValidationFailure.Reason? = null)
        : VerificationValidationViewModel()

    data class Validating(
        override val lineNumber: String,
        override val id: String,
        override val pin: String)
        : VerificationValidationViewModel()

    data class ValidationSuccess(
        override val lineNumber: String,
        override val id: String,
        override val pin: String)
        : VerificationValidationViewModel()

    data class ValidationFailure @JvmOverloads constructor(
        override val lineNumber: String,
        override val id: String,
        override val pin: String,
        val reason: Reason = Reason())
        : VerificationValidationViewModel() {

        constructor(lineNumber: String, id: String, pin: String, message: String? = null)
                : this(lineNumber, id, pin, Reason(message))

        data class Reason @JvmOverloads constructor(val message: String? = null)
    }
}
